@@ -97,14 +97,16 @@ chxj_encoding(request_rec *r, const char *src, apr_size_t *len)
     if (result == (size_t)(-1)) {
       if (E2BIG == errno) {
         ERR(r, "There is not sufficient room at *outbuf.");
+        break;
       }
       else if (EILSEQ == errno) {
-        ERR(r, "An invalid multibyte sequence has been encountered in the input. input:[%s]", ibuf);
+        ERR(r, "%s:%d An invalid multibyte sequence has been encountered in the input. input:[%s]", __FILE__,__LINE__,ibuf);
+        chxj_convert_illegal_charactor_sequence(r, entryp, &ibuf, &ilen, &obuf, &olen);
       }
       else if (EINVAL == errno) {
         ERR(r, "An incomplete multibyte sequence has been encountered in the input. input:[%s]", ibuf);
+        break;
       }
-      break;
     }
   }
   *len = strlen(spos);
@@ -115,6 +117,118 @@ chxj_encoding(request_rec *r, const char *src, apr_size_t *len)
 }
 
 
+void
+chxj_convert_illegal_charactor_sequence(request_rec *r, chxjconvrule_entry  *entryp, char **ibuf, apr_size_t *ilen, char **obuf, apr_size_t *olen)
+{
+  if (STRCASEEQ('u','U',"UTF-8", entryp->encoding) || STRCASEEQ('u','U',"UTF8", entryp->encoding)) {
+    if ((0xe0 & **ibuf) == 0xc0) {
+      /* 2byte charactor */
+      **obuf = '?';
+      *obuf += 1;
+      *olen -= 1;
+      *ibuf += 2;
+      DBG(r, "passed 2byte.");
+    }
+    else if ((0xf0 & **ibuf) == 0xe0) {
+      /* 3byte charactor */
+      **obuf = '?';
+      *obuf += 1;
+      *olen -= 1;
+      *ibuf +=3;
+      DBG(r, "passed 3byte.");
+    }
+    else if ((0xf8 & **ibuf) == 0xf0) {
+      /* 4byte charactor */
+      **obuf = '?';
+      *obuf += 1;
+      *olen -= 1;
+      *ibuf +=4;
+      DBG(r, "passed 4byte.");
+    }
+    else if ((0xc0 & **ibuf) == 0x80) {
+      /* 1byte charactor */
+      **obuf = '?';
+      *obuf += 1;
+      *olen -= 1;
+      *ibuf += 1;
+      DBG(r, "passed 1byte.");
+    }
+    else {
+      /* unknown charactor */
+      **obuf = '?';
+      *obuf += 1;
+      *olen -= 1;
+      *ibuf += 1;
+      DBG(r, "passed 1byte.");
+    }
+  }
+  else if (STRCASEEQ('e','E', "EUCJP",               entryp->encoding)
+      ||   STRCASEEQ('c','C', "CSEUCPKDFMTJAPANESE", entryp->encoding)
+      ||   STRCASEEQ('e','E', "EUC-JISX0213",        entryp->encoding)
+      ||   STRCASEEQ('e','E', "EUC-JP-MS",           entryp->encoding)
+      ||   STRCASEEQ('e','E', "EUC-JP",              entryp->encoding)
+      ||   STRCASEEQ('e','E', "EUCJP-MS",            entryp->encoding)
+      ||   STRCASEEQ('e','E', "EUCJP-OPEN",          entryp->encoding)
+      ||   STRCASEEQ('e','E', "EUCJP-WIN",           entryp->encoding)
+      ||   STRCASEEQ('e','E', "EUCJP",               entryp->encoding)) {
+    if ((unsigned char)**ibuf == 0x8F) {
+      /* 3byte charactor */
+      **obuf = '?';
+      *obuf += 1;
+      *olen -= 1;
+      *ibuf +=3;
+      DBG(r, "passed 3byte.");
+    }
+    else {
+      /* 2byte charactor */
+      **obuf = '?';
+      *obuf += 1;
+      *olen -= 1;
+      *ibuf += 2;
+      DBG(r, "passed 2byte.");
+    }
+  }
+  else if (STRCASEEQ('c', 'C', "CP932",     entryp->encoding)
+      ||   STRCASEEQ('c', 'C', "CSIBM932",  entryp->encoding)
+      ||   STRCASEEQ('i', 'I', "IBM-932",   entryp->encoding)
+      ||   STRCASEEQ('i', 'I', "IBM932",    entryp->encoding)
+      ||   STRCASEEQ('m', 'M', "MS932",     entryp->encoding)
+      ||   STRCASEEQ('m', 'M', "MS_KANJI",  entryp->encoding)
+      ||   STRCASEEQ('s', 'S', "SJIS-OPEN", entryp->encoding)
+      ||   STRCASEEQ('s', 'S', "SJIS-WIN",  entryp->encoding)
+      ||   STRCASEEQ('s', 'S', "SJIS",      entryp->encoding)) {
+    if ( ( ((0x81 <= (unsigned char)**ibuf) && (0x9f >= (unsigned char)**ibuf))
+        || ((0xe0 <= (unsigned char)**ibuf) && (0xfc >= (unsigned char)**ibuf)))
+       &&
+       (  ((0x40 <= (unsigned char)*((*ibuf)+1)) && (0x7e >= (unsigned char)*((*ibuf)+1)))
+        ||((0x80 <= (unsigned char)*((*ibuf)+1)) && (0xfc >= (unsigned char)*((*ibuf)+1))))) {
+      /* 2byte charactor */
+      **obuf = '?';
+      *obuf += 1;
+      *olen -= 1;
+      *ibuf += 2;
+      DBG(r, "passed 2byte.");
+    }
+    else {
+      /* 1byte charactor */
+      **obuf = '?';
+      *obuf += 1;
+      *olen -= 1;
+      *ibuf += 1;
+      DBG(r, "passed 1byte.");
+    }
+  }
+  else {
+    *ilen = 0;
+    return;
+  }
+  if (ibuf && *ibuf) {
+    *ilen = strlen(*ibuf);
+    DBG(r, "new len = [%d].", *ilen);
+  }
+}
+
+
 char *
 chxj_rencoding(request_rec *r, const char *src, apr_size_t *len)
 {
@@ -190,14 +304,16 @@ chxj_rencoding(request_rec *r, const char *src, apr_size_t *len)
     if (result == (size_t)(-1)) {
       if (E2BIG == errno) {
         ERR(r, "There is not sufficient room at *outbuf");
+        break;
       }
       else if (EILSEQ == errno) {
         ERR(r, "An invalid multibyte sequence has been encountered in the input. input:[%s]", ibuf);
+        chxj_convert_illegal_charactor_sequence(r, entryp, &ibuf, &ilen, &obuf, &olen);
       }
       else if (EINVAL == errno) {
         ERR(r, "An incomplete multibyte sequence has been encountered in the input. input:[%s]", ibuf);
+        break;
       }
-      break;
     }
   }
 
